@@ -1,6 +1,5 @@
-use ormlite_core::model::{BuildsQueryBuilder, PartialModel, TableMeta};
+use ormlite_core::model::{HasQueryBuilder, Insertable, ModelBuilder, TableMeta};
 use ormlite_core::{BoxFuture, Error, Result, SelectQueryBuilder};
-use sqlx::Database;
 
 pub static PLACEHOLDER: &str = "?";
 pub static CREATE_TABLE_SQL: &str =
@@ -32,34 +31,11 @@ impl ormlite_core::model::TableMeta for Person {
     }
 }
 
-/*
-impl Person {
-    fn upsert(self, db: &mut <DB as Database>::Connection) -> BoxFuture<Result<Self>> {
-        Box::pin(async move {
-            let q = format!(
-                "INSERT INTO {} ({}) VALUES ({}) ON CONFLICT ({}) DO UPDATE SET {}\
-                RETURNING *",
-                Self::table_name(),
-                Self::fields().join(", "),
-                (0..Self::num_fields())
-                    .map(|_| PLACEHOLDER)
-                    .collect::<Vec<_>>()
-                    .join(", "),
-            );
-            sqlx::query_as::<_, Self>(&q)
-                .bind(self.id)
-                .bind(self.name)
-                .bind(self.age)
-                .fetch_one(db)
-                .await
-                .map_err(Error::from)
-        })
-    }
-}
-*/
-
 impl crate::model::Model<DB> for Person {
-    fn insert(self, db: &mut <DB as Database>::Connection) -> BoxFuture<Result<Self>> {
+    fn insert<'e, E>(self, db: E) -> BoxFuture<'e, Result<Self>>
+    where
+        E: 'e + sqlx::Executor<'e, Database = DB>,
+    {
         Box::pin(async move {
             let q = format!(
                 "INSERT INTO {} ({}) VALUES ({}) RETURNING *",
@@ -79,8 +55,10 @@ impl crate::model::Model<DB> for Person {
                 .map_err(Error::from)
         })
     }
-
-    fn update_all_fields(self, db: &mut <DB as Database>::Connection) -> BoxFuture<Result<Self>> {
+    fn update_all_fields<'e, E>(self, db: E) -> BoxFuture<'e, Result<Self>>
+    where
+        E: 'e + sqlx::Executor<'e, Database = DB>,
+    {
         Box::pin(async move {
             let q = format!(
                 "UPDATE {} SET {} WHERE {} = {} RETURNING *",
@@ -99,7 +77,10 @@ impl crate::model::Model<DB> for Person {
         })
     }
 
-    fn delete(self, db: &mut <DB as sqlx::Database>::Connection) -> BoxFuture<Result<()>> {
+    fn delete<'e, E>(self, db: E) -> BoxFuture<'e, Result<()>>
+    where
+        E: 'e + sqlx::Executor<'e, Database = DB>,
+    {
         Box::pin(async move {
             let q = format!(
                 "DELETE from {} WHERE {} = {}",
@@ -116,12 +97,11 @@ impl crate::model::Model<DB> for Person {
         })
     }
 
-    fn get_one<'db, 'arg: 'db, T>(
-        id: T,
-        db: &'db mut <DB as Database>::Connection,
-    ) -> BoxFuture<'db, Result<Self>>
+    fn get_one<'e, 'a, Arg, E>(id: Arg, db: E) -> BoxFuture<'e, Result<Self>>
     where
-        T: 'arg + Send + for<'r> sqlx::Encode<'r, DB> + sqlx::Type<DB>,
+        'a: 'e,
+        E: 'e + sqlx::Executor<'e, Database = DB>,
+        Arg: 'a + Send + for<'r> sqlx::Encode<'r, DB> + sqlx::Type<DB>,
     {
         let text = format!(
             "SELECT * FROM {} WHERE {} = {}",
@@ -145,15 +125,14 @@ impl crate::model::Model<DB> for Person {
     }
 }
 
-// done
-impl BuildsQueryBuilder<DB, Box<dyn Iterator<Item = String>>> for Person {
-    fn select<'args>() -> SelectQueryBuilder<'args, DB, Self, Box<dyn Iterator<Item = String>>> {
+impl HasQueryBuilder<DB, Box<dyn Iterator<Item = String>>> for Person {
+    fn select<'a>() -> SelectQueryBuilder<'a, DB, Self, Box<dyn Iterator<Item = String>>> {
         SelectQueryBuilder::default().column(&format!("{}.*", Self::table_name()))
     }
 }
 
 // done
-impl<'a> ormlite_core::model::BuildsPartialModel<'a, PartialPerson<'a>> for Person {
+impl<'a> ormlite_core::model::HasModelBuilder<'a, PartialPerson<'a>> for Person {
     fn build() -> PartialPerson<'a> {
         PartialPerson::default()
     }
@@ -165,7 +144,6 @@ impl<'a> ormlite_core::model::BuildsPartialModel<'a, PartialPerson<'a>> for Pers
     }
 }
 
-// done
 pub struct PartialPerson<'a> {
     id: Option<u32>,
     name: Option<String>,
@@ -174,7 +152,6 @@ pub struct PartialPerson<'a> {
     updating: Option<&'a Person>,
 }
 
-// done
 impl<'a> Default for PartialPerson<'a> {
     fn default() -> Self {
         PartialPerson {
@@ -186,7 +163,6 @@ impl<'a> Default for PartialPerson<'a> {
     }
 }
 
-// done
 impl<'a> PartialPerson<'a> {
     pub fn id(mut self, id: u32) -> Self {
         self.id = Some(id);
@@ -218,14 +194,13 @@ impl<'a> PartialPerson<'a> {
     }
 }
 
-// done
-impl<'a> PartialModel<'a, DB> for PartialPerson<'a> {
+impl<'a> ModelBuilder<'a, DB> for PartialPerson<'a> {
     type Model = Person;
 
-    fn insert<'db: 'a>(
-        self,
-        db: &'db mut <DB as sqlx::Database>::Connection,
-    ) -> BoxFuture<'a, Result<Self::Model>> {
+    fn insert<'e: 'a, E>(self, db: E) -> BoxFuture<'a, Result<Self::Model>>
+    where
+        E: 'e + sqlx::Executor<'e, Database = DB>,
+    {
         Box::pin(async move {
             let insert_fields = self.modified_fields();
             let query = format!(
@@ -252,10 +227,10 @@ impl<'a> PartialModel<'a, DB> for PartialPerson<'a> {
         })
     }
 
-    fn update<'db: 'a>(
-        self,
-        db: &'db mut <DB as sqlx::Database>::Connection,
-    ) -> BoxFuture<'a, Result<Self::Model>> {
+    fn update<'e: 'a, E>(self, db: E) -> BoxFuture<'a, Result<Self::Model>>
+    where
+        E: 'e + sqlx::Executor<'e, Database = DB>,
+    {
         Box::pin(async move {
             let update_fields = self.modified_fields();
             let text = format!(
@@ -263,7 +238,7 @@ impl<'a> PartialModel<'a, DB> for PartialPerson<'a> {
                 Self::Model::table_name(),
                 update_fields.into_iter().map(|col| format!("{} = {}", col, PLACEHOLDER)).collect::<Vec<_>>().join(", "),
                 Self::Model::primary_key_column(),
-                self.updating.expect("Tried to call PartialModel::update(), but no model found to update. Call should look something like: <Model>.update_partial().update(&mut db)").id,
+                self.updating.expect("Tried to call ModelBuilder::update(), but no model found to update. Call should look something like: <Model>.update_partial().update(&mut db)").id,
             );
             let mut q = sqlx::query_as::<DB, Self::Model>(&text);
             if let Some(value) = self.id {
@@ -279,3 +254,35 @@ impl<'a> PartialModel<'a, DB> for PartialPerson<'a> {
         })
     }
 }
+
+pub struct InsertPerson {
+    pub name: String,
+    pub age: u8,
+}
+
+impl<'a> Insertable<'a, DB> for InsertPerson {
+    type Model = Person;
+
+    fn insert<'e: 'a, E>(self, db: E) -> BoxFuture<'a, Result<Self::Model>>
+    where
+        E: 'e + sqlx::Executor<'e, Database = DB>,
+    {
+        Box::pin(async move {
+            let insert_fields = ["name", "age"];
+            let query = format!(
+                "INSERT INTO {} ({}) VALUES ({}) RETURNING *",
+                Self::Model::table_name(),
+                insert_fields.join(", "),
+                insert_fields
+                    .iter()
+                    .map(|_| PLACEHOLDER)
+                    .collect::<Vec<_>>()
+                    .join(", "),
+            );
+            let mut q = sqlx::query_as::<DB, Self::Model>(&query);
+            q = q.bind(self.name);
+            q = q.bind(self.age);
+            q.fetch_one(db).await.map_err(Error::from)
+        })
+    }
+}
